@@ -14,7 +14,7 @@ use Getopt::Std;
 use JSON;
 use Net::Shadowsocks::Server;
 
-my $version = "0.9.0";
+my $version = "0.9.1";
 
 sub main::HELP_MESSAGE()
 {
@@ -31,11 +31,15 @@ sub main::HELP_MESSAGE()
     print("\tcamellia-128-cfb camellia-128-ctr camellia-128-ofb\n");
     print("\tcamellia-192-cfb camellia-192-ctr camellia-192-ofb\n");
     print("\tcamellia-256-cfb camellia-256-ctr camellia-256-ofb\n");
-    print("\tchacha20-ietf chacha20-ietf-poly1305\n");
-    print("\trc4-md5 rc4-sha\n");
+    if ($^O ne "MSWin32")
+    {
+        print("\tchacha20-ietf chacha20-ietf-poly1305\n");
+    }
+    print("\trabbit\n");
     print("\trc6-128-cfb rc6-128-ctr rc6-128-ofb\n");
     print("\trc6-192-cfb rc6-192-ctr rc6-192-ofb\n");
     print("\trc6-256-cfb rc6-256-ctr rc6-256-ofb\n");
+    print("\tspritz\n");
     printf("\t-c CONFIG\tName of your config.json file with full path\n");
     print("\t-h, --help\tDisplay help information\n");
     print("\t-v, --version\tDisplay version information\n");
@@ -103,7 +107,7 @@ ssserver.pl
 
 =head1 VERSION
 
-Version 0.9.0
+Version 0.9.1
 
 =head1 SYNOPSIS
 
@@ -123,10 +127,11 @@ Usage: ssserver.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
 	camellia-192-cfb camellia-192-ctr camellia-192-ofb
 	camellia-256-cfb camellia-256-ctr camellia-256-ofb
 	chacha20-ietf chacha20-ietf-poly1305
-	rc4-md5 rc4-sha
+	rabbit
 	rc6-128-cfb rc6-128-ctr rc6-128-ofb
 	rc6-192-cfb rc6-192-ctr rc6-192-ofb
 	rc6-256-cfb rc6-256-ctr rc6-256-ofb
+	spritz
 	-c CONFIG	Name of your config.json file with full path
 	-h, --help	Display help information
 	-v, --version	Display version information
@@ -145,24 +150,28 @@ Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭)
 protocol client and server , Shadowsocks is a secure transport protocol based on
 SOCKS Protocol Version 5 (RFC 1928 ).
 
-1. A total of 28 encryption methods are supported:
+1. A total of 34 encryption methods are supported:
 
-aes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb
-aes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb
-aes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb
-camellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb
-camellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb
-camellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb
-chacha20-ietf
-rc4-md5 rc4-sha rc6
+        aes-128-cfb aes-128-ctr aes-128-gcm aes-128-ofb
+	aes-192-cfb aes-192-ctr aes-192-gcm aes-192-ofb
+	aes-256-cfb aes-256-ctr aes-256-gcm aes-256-ofb
+	camellia-128-cfb camellia-128-ctr camellia-128-ofb
+	camellia-192-cfb camellia-192-ctr camellia-192-ofb
+	camellia-256-cfb camellia-256-ctr camellia-256-ofb
+	chacha20-ietf chacha20-ietf-poly1305
+	rabbit
+	rc6-128-cfb rc6-128-ctr rc6-128-ofb
+	rc6-192-cfb rc6-192-ctr rc6-192-ofb
+	rc6-256-cfb rc6-256-ctr rc6-256-ofb
+	spritz
 
 2.The following ciphers deprecated by Shadowsocks are not supported:
-bf-cfb chacha20 salsa20
+
+        bf-cfb chacha20 salsa20 rc4-md5
 
 3.The following ciphers recommended by Shadowsocks are not supported yet:
-aes-128-gcm aes-192-gcm aes-256-gcm
-chacha20-ietf-poly1305
-xchacha20-ietf-poly1305
+
+        xchacha20-ietf-poly1305
 
 Please note TLS 1.2 has removed IDEA and DES cipher suites. and because of
 CVE-2016-2183,  http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2016-2183
