@@ -5,6 +5,18 @@ package Net::Shadowsocks;
 use 5.006;
 use strict;
 use warnings;
+BEGIN 
+{
+       my $_win32 = ($^O eq "MSWin32") ? 1 : 0;                 
+       if ($_win32) 
+       {
+       }
+       else 
+       {
+           eval "use Crypt::NaCl::Sodium";
+           die "$@" if $@;
+       }
+}
 use AnyEvent::Log;
 use Carp;
 use Crypt::AuthEnc::GCM;
@@ -15,14 +27,17 @@ use Crypt::KeyDerivation qw(hkdf);
 use Crypt::Mode::CFB;
 use Crypt::Mode::CTR;
 use Crypt::Mode::OFB;
-use Crypt::NaCl::Sodium;
+#use Crypt::NaCl::Sodium;
+use Crypt::Rabbit;
 use Crypt::Random qw( makerandom_octet );
+#use Crypt::RC4::XS;
+use Crypt::Spritz;
 use Digest::MD5;
 use Digest::SHA;
-use Mcrypt qw(:FUNCS);
+#use Mcrypt qw(:FUNCS);
 #use String::HexConvert ':all';
 
-our $VERSION = '0.9.0';
+our $VERSION = '0.9.1';
 
 our %_ciphers = 
     (
@@ -58,8 +73,9 @@ our %_ciphers =
         "camellia-256-ofb" => ['Camellia','ofb',32,16],
         "chacha20-ietf" => [undef,undef,32,12],
         "chacha20-ietf-poly1305" =>  [undef,undef,32,12],
-         "rc4-md5"  => ['arcfour','stream',16,16],
-        "rc4-sha" => ['arcfour','stream',16,20],
+        "rabbit" => ['rabbit','stream',16,16],        
+        #"rc4-md5"  => ['arcfour','stream',16,16],
+        #"rc4-sha" => ['arcfour','stream',16,20],
         "rc6-128-cfb" => ['RC6','cfb',16,16],
         "rc6-128-ctr" => ['RC6','ctr',16,16],
         "rc6-128-ofb" => ['RC6','ofb',16,16],
@@ -69,6 +85,7 @@ our %_ciphers =
         "rc6-256-cfb" => ['RC6','cfb',32,16],
         "rc6-256-ctr" => ['RC6','ctr',32,16],
         "rc6-256-ofb" => ['RC6','ofb',32,16],
+        "spritz" =>  ['spritz','stream',16,16], 
     );
 
 sub _EVP_BytesToKey($$$$$) 
@@ -125,56 +142,52 @@ sub _initialize_cipher($$)
     my $_decrypt_nonce;
     if (!defined($_method))
     {
-        AE::log info => "Encryption method undefinde, using RC4-MD5." ;
-        $_encryptor = Mcrypt::mcrypt_load('arcfour','','stream' , '' );
-        $_decryptor = Mcrypt::mcrypt_load('arcfour' ,'','stream', '' );
+        AE::log info => "Encryption method undefinde, using spritz." ;
+
         $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
         $_iv = makerandom_octet( Length => 16 );
-        my $md = Digest::MD5->new();
-        $md->add($_key . $_iv);
-        my $encrypt_rc4_key = $md->digest();
-        Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );      
-        }
-        else
+        $_encryptor = Crypt::Spritz::Cipher->new($_key, $_iv);
+    }
+    else
+    {
+        if($_method =~ /^chacha20/)
         {
-            if($_method =~ /^chacha20/)
+            if ($_method eq 'chacha20-ietf')
             {
-                 if ($_method eq 'chacha20-ietf')
-                 {
-                    $_encryptor = Crypt::NaCl::Sodium->stream();
-                    #$_key = $_encryptor->chacha20_keygen();
-                    $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
-                    $_encrypt_nonce = $_encryptor->chacha20_ietf_nonce();
-                    $_decryptor = Crypt::NaCl::Sodium->stream();
-                    $_iv = $_encrypt_nonce;
-                }
-                else
-                {
-                    $_encryptor = Crypt::NaCl::Sodium->aead();
-                    #$_key = $_encryptor->chacha20_keygen();
-                    $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
-                    my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
-                    $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
-                    #carp ascii_to_hex($_encrypt_subkey);
-                    $_decryptor = Crypt::NaCl::Sodium->aead();
-                    $_iv = $_encrypt_salt;
-                    $_encrypt_nonce = $_encryptor->ietf_nonce("\0");
-                    $_decrypt_nonce = $_decryptor->ietf_nonce("\0");
-                    #carp $_encrypt_nonce;
-                    #carp ascii_to_hex($_encrypt_nonce);
-                }
+                $_encryptor = Crypt::NaCl::Sodium->stream();
+                #$_key = $_encryptor->chacha20_keygen();
+                $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
+                $_encrypt_nonce = $_encryptor->chacha20_ietf_nonce();
+                $_decryptor = Crypt::NaCl::Sodium->stream();
+                $_iv = $_encrypt_nonce;
             }
-             elsif($_method =~ /gcm$/ )
+            else
             {
-                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_encryptor = Crypt::NaCl::Sodium->aead();
+                #$_key = $_encryptor->chacha20_keygen();
+                $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
                 my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
-                my $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
-                $_iv = $_encrypt_salt; 
-                $_encryptor = Crypt::AuthEnc::GCM->new('AES', $_encrypt_subkey);
-                $_encrypt_nonce = Data::BytesLocker->new("\0" x 12);
-                $_decrypt_nonce = Data::BytesLocker->new("\0" x 12);
+                $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
+                #carp ascii_to_hex($_encrypt_subkey);
+                $_decryptor = Crypt::NaCl::Sodium->aead();
+                $_iv = $_encrypt_salt;
+                $_encrypt_nonce = $_encryptor->ietf_nonce("\0");
+                $_decrypt_nonce = $_decryptor->ietf_nonce("\0");
+                #carp $_encrypt_nonce;
+                #carp ascii_to_hex($_encrypt_nonce);
             }
-           
+        }
+        elsif($_method =~ /gcm$/ )
+        {
+            $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+            my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
+            my $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
+            $_iv = $_encrypt_salt; 
+            $_encryptor = Crypt::AuthEnc::GCM->new('AES', $_encrypt_subkey);
+            $_encrypt_nonce = Data::BytesLocker->new("\0" x 12);
+            $_decrypt_nonce = Data::BytesLocker->new("\0" x 12);
+        }
+=cut for comment           
             elsif($_method =~ /^rc4/ )
             {
                 $_encryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0],'',$_ciphers{$_method}->[1] , '' );
@@ -197,10 +210,26 @@ sub _initialize_cipher($$)
                     }
                     Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );
             }
-             
-            elsif( ($_method =~ /^aes/  or $_method =~ /^camellia/ or $_method =~ /^rc6/ ) and $_method !~ /gcm$/ )
-            {
-                          if($_method =~ /cfb$/)
+=cut            
+        elsif($_method eq 'rabbit')
+        {
+            $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+            $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+            my $md = Digest::MD5->new();
+            $md->add($_key . $_iv);
+            my $encrypt_rabbit_key = $md->digest();
+            $_encryptor = Crypt::Rabbit->new($encrypt_rabbit_key);
+            #$_decryptor = Crypt::Rabbit->new($encrypted_rabbit_key);
+        }
+        elsif($_method eq "spritz") 
+        {
+            $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
+            $_iv = makerandom_octet( Length => 16 );
+            $_encryptor = Crypt::Spritz::Cipher->new($_key, $_iv);
+        }
+        elsif( ($_method =~ /^aes/  or $_method =~ /^camellia/ or $_method =~ /^rc6/ ) and $_method !~ /gcm$/ )
+        {
+            if($_method =~ /cfb$/)
             {
                 $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
                 $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
@@ -329,7 +358,7 @@ Net::Shadowsocks - the asynchronous, non-blocking shadowsocks client and server.
 
 =head1 VERSION
 
-Version 0.9.0
+Version 0.9.1
 
 =head1 SYNOPSIS
     
@@ -346,14 +375,15 @@ Shadowsocks is a secure transport protocol based on SOCKS Protocol Version 5 (RF
 	camellia-192-cfb camellia-192-ctr camellia-192-ofb
 	camellia-256-cfb camellia-256-ctr camellia-256-ofb
 	chacha20-ietf chacha20-ietf-poly1305
-	rc4-md5 rc4-sha
+	rabbit
 	rc6-128-cfb rc6-128-ctr rc6-128-ofb
 	rc6-192-cfb rc6-192-ctr rc6-192-ofb
 	rc6-256-cfb rc6-256-ctr rc6-256-ofb
+	spritz
 
 2.The following ciphers deprecated by Shadowsocks are not supported: 
 
-      bf-cfb chacha20 salsa20
+      bf-cfb chacha20 salsa20 rc4-md5
 
 3.The following ciphers recommended by Shadowsocks are not supported yet: 
  
