@@ -12,16 +12,27 @@ package Net::Shadowsocks::Client;
     use Carp;
     use Config;
     use Crypt::KeyDerivation qw(hkdf);
-    use Crypt::NaCl::Sodium qw( :utils );
+    BEGIN 
+    {
+       my $_win32 = ($^O eq "MSWin32") ? 1 : 0;                 
+       if ($_win32) 
+       {
+       }
+       else 
+       {
+           eval "use Crypt::NaCl::Sodium qw( :utils )";
+           die "$@" if $@;
+       }
+    }
     use Digest::MD5;
     use Digest::SHA;
     use IO::Socket::Socks qw(:constants);
     use Net::Shadowsocks;
     use Net::Shadowsocks qw( _get_algorithm _get_mode _get_iv_size _EVP_BytesToKey _initialize_cipher);
     use Socket qw(IPPROTO_TCP MSG_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
-    use String::HexConvert ':all';
+    #use String::HexConvert ':all';
     
-    our $VERSION = '0.9.0';
+    our $VERSION = '0.9.1';
     $AnyEvent::Log::FILTER->level ("info");
     sub new($$$$$$) 
     {
@@ -179,7 +190,15 @@ package Net::Shadowsocks::Client;
                                             on_prepare => sub
                                             {
                                                 my $phandle = shift;
-                                                setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                                if ($^O eq "MSWin32")
+                                                {
+                                                     setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEADDR, 1);
+                                                }
+                                                else
+                                                {
+                                                    setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEADDR, 1);
+                                                    setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                                }
                                             },
                                             on_read => sub 
                                             {
@@ -197,6 +216,7 @@ package Net::Shadowsocks::Client;
                                                         $server_iv = substr( $incoming_data, 0,length($iv) );
                                                         $incoming_data = substr($incoming_data,length($iv));
                                                         $rhandle->{rbuf} = $incoming_data;
+=cut for comment                                                        
                                                         if ( !defined($self->{method}) or $self->{method} eq'rc4-md5' )
                                                         {
                                                             my $md = Digest::MD5->new();
@@ -213,6 +233,20 @@ package Net::Shadowsocks::Client;
                                                             my $decrypt_rc4_key = substr($sha->digest(),0,16);
                                                             Mcrypt::mcrypt_init($decryptor,$decrypt_rc4_key,'' );
                                                         }
+=cut                                                     
+                                                        if ( !defined($self->{method}) or $self->{method} eq'spritz' )
+                                                        {
+                                                           
+                                                            $decryptor = Crypt::Spritz::Cipher->new($key,$server_iv);
+                                                        } 
+                                                        elsif ( $self->{method} eq'rabbit' )
+                                                        {
+                                                            my $md = Digest::MD5->new();
+                                                            $md->add($key . $server_iv);
+                                                            #$md->add($server_iv);
+                                                            my $decrypt_rabbit_key = $md->digest();
+                                                            $decryptor = Crypt::Rabbit->new($decrypt_rabbit_key);
+                                                        }   
                                                         elsif($self->{method} =~ /gcm$/ or $self->{method} =~ /poly1305$/)
                                                         {
                                                           $decrypt_subkey = hkdf($key,$server_iv,'SHA1',Net::Shadowsocks::_get_key_size($self->{method}),"ss-subkey");
@@ -230,12 +264,16 @@ package Net::Shadowsocks::Client;
                                                 }
                                                 else
                                                 {
-
+=cut for comment
                                                 if (  $self->{method} =~ /^rc4/)
                                                 {
                                                     $decrypted_data = Mcrypt::mcrypt_decrypt($decryptor, $incoming_data );
                                                 }
-
+=cut
+                                                    if (  $self->{method} eq "rabbit" or $self->{method} eq "spritz")
+                                                    {
+                                                        $decrypted_data = $decryptor->decrypt($incoming_data);
+                                                    }
                                                 elsif($self->{method} =~ /^chacha20/)
                                                 {
                                                     if ($self->{method} eq 'chacha20-ietf')
@@ -410,11 +448,16 @@ package Net::Shadowsocks::Client;
                                     $plain_data = $addr_to_send . $client_buffer;
                                     $addr_to_send = '';
                                 }
-
+=cut for comment
                                 if ( $self->{method} =~ /^rc4/  ) 
                                 {
                                     $encrypted_data = Mcrypt::mcrypt_encrypt($encryptor,$plain_data);                                
                                 }
+=cut
+                                if ( $self->{method} eq "rabbit" or $self->{method} eq "spritz" ) 
+                                {
+                                    $encrypted_data = $encryptor->encrypt($plain_data);                                
+                                }
                                 elsif($self->{method} =~ /^chacha20/ )
                                 {
                                     if($self->{method} eq 'chacha20-ietf')
@@ -521,7 +564,7 @@ Net::Shadowsocks::Client - Shadowsocks protocol client module.
 
 =head1 VERSION
 
-Version 0.9.0
+Version 0.9.1
 
 =head1 SYNOPSIS
 
@@ -538,14 +581,15 @@ Version 0.9.0
 	camellia-192-cfb camellia-192-ctr camellia-192-ofb
 	camellia-256-cfb camellia-256-ctr camellia-256-ofb
 	chacha20-ietf chacha20-ietf-poly1305
-	rc4-md5 rc4-sha
+	rabbit
 	rc6-128-cfb rc6-128-ctr rc6-128-ofb
 	rc6-192-cfb rc6-192-ctr rc6-192-ofb
 	rc6-256-cfb rc6-256-ctr rc6-256-ofb
+	spritz
 
 2.The following ciphers deprecated by Shadowsocks are not supported: 
 
-      bf-cfb chacha20 salsa20
+      bf-cfb chacha20 salsa20 rc4-md5
 
 3.The following ciphers recommended by Shadowsocks are not supported yet: 
  
