@@ -1,6 +1,6 @@
  package Net::Shadowsocks::Server;
 
-    # ABSTRACT: Shadowsocks protocol server.
+    # ABSTRACT: Shadowsocks protocol server module.
     # KEYWORDS: socks shadowsocks rfc1928 Great Firewall Internet censorship in China
     use 5.006;
     use strict;
@@ -13,15 +13,26 @@
     use Config;
     use Crypt::KeyDerivation;
     use Crypt::KeyDerivation qw(hkdf);
-    use Crypt::NaCl::Sodium qw( :utils );
+    BEGIN 
+    {
+       my $_win32 = ($^O eq "MSWin32") ? 1 : 0;                 
+       if ($_win32) 
+       {
+       }
+       else 
+       {
+           eval "use Crypt::NaCl::Sodium qw( :utils )";
+           die "$@" if $@;
+       }
+    }
     use Digest::MD5;
     use Digest::SHA;
     use Net::Shadowsocks;
     use Net::Shadowsocks qw(_get_algorithm _get_mode _get_key_size _EVP_BytesToKey _initialize_cipher);
     use Socket qw(IPPROTO_TCP TCP_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
-    use String::HexConvert ':all';
+    #use String::HexConvert ':all';
     
-    our $VERSION = '0.9.0';
+    our $VERSION = '0.9.1';
     
     $AnyEvent::Log::FILTER->level ("info");
     
@@ -99,6 +110,7 @@
                                     $client_iv = substr( $client_buffer, 0, length($iv) );
                                     $incoming_data = substr( $client_buffer, length($iv) );
                                     $chandle->{rbuf} = $incoming_data;
+=cut for comment
                                     if ( !defined($self->{method}) or $self->{method} eq 'rc4-md5' ) 
                                     {
                                         my $md = Digest::MD5->new();
@@ -115,6 +127,19 @@
                                         my $decrypt_rc4_key = substr($sha->digest(),0,16);
                                         Mcrypt::mcrypt_init($decryptor, $decrypt_rc4_key, '' );
                                     }
+=cut
+                                    if ( !defined($self->{method}) or $self->{method} eq 'spritz' ) 
+                                    {
+                                        $decryptor = Crypt::Spritz::Cipher->new($key,$client_iv);
+                                    }
+                                    elsif ( $self->{method} eq 'rabbit' ) 
+                                    {
+                                        my $md = Digest::MD5->new();
+                                        $md->add($key .$client_iv );
+                                        #$md->add($client_iv);
+                                        my $decrypt_rabbit_key = $md->digest();
+                                        $decryptor = Crypt::Rabbit->new($decrypt_rabbit_key);
+                                    }
                                     elsif($self->{method} =~ /gcm$/ or $self->{method} =~ /poly1305$/)
                                     {
                                         $decrypt_subkey = hkdf($key,$client_iv,'SHA1',Net::Shadowsocks::_get_key_size($self->{method}),"ss-subkey");
@@ -134,10 +159,16 @@
                              {      
                                  if ($mode == 0)
                                  {
+=cut for comment
                                      if ($self->{method} =~ /^rc4/ )
                                      {
                                           $decrypted_data = Mcrypt::mcrypt_decrypt($decryptor, $incoming_data );
                                      }
+=cut
+                                     if ($self->{method} eq "rabbit" or $self->{method} eq "spritz")
+                                     {
+                                          $decrypted_data = $decryptor->decrypt($incoming_data );
+                                     }
                                      elsif($self->{method} =~ /^chacha20/)
                                      {
                                         if ($self->{method} eq 'chacha20-ietf')
@@ -372,7 +403,15 @@
                                                   on_prepare => sub
                                                   {
                                                       my $phandle = shift;
-                                                      setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                                      if ($^O eq "MSWin32")
+                                                      {
+                                                           setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEADDR, 1);
+                                                      }
+                                                      else
+                                                      {
+                                                          setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEADDR, 1);
+                                                          setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                                      }
                                                   },
                                                   on_read => sub 
                                                   {
@@ -380,12 +419,16 @@
                                                       my $remote_buffer = $rhandle->rbuf();
                                                       my $plain_data = $remote_buffer;
                                                       my $encrypted_data;
-                                                
+=cut for comment                                                
                                                       if ( $self->{method} =~ /^rc4/)
                                                       {
                                                           $encrypted_data = Mcrypt::mcrypt_encrypt($encryptor, $plain_data );    
                                                       }
-
+=cut
+                                                      if ( $self->{method} eq "rabbit" or $self->{method} eq "spritz")
+                                                      {
+                                                          $encrypted_data = $encryptor->encrypt($plain_data );    
+                                                      }
                                                       elsif($self->{method} =~ /^chacha20/)
                                                       {
                                                           if($self->{method} eq 'chacha20-ietf')
@@ -460,10 +503,16 @@
                                }
                                else
                                {
+=cut for comment
                                      if ($self->{method} =~ /^rc4/ )
                                      {
                                           $decrypted_data = Mcrypt::mcrypt_decrypt($decryptor, $incoming_data );
                                      }
+=cut
+                                     if ($self->{method} eq "rabbit" or $self->{method} eq "spritz")
+                                     {
+                                          $decrypted_data = $decryptor->decrypt($incoming_data );
+                                     }
                                      elsif($self->{method} =~ /^chacha20/)
                                      {
                                         if ($self->{method} eq 'chacha20-ietf')
@@ -657,7 +706,7 @@ Net::Shadowsocks::Server - Shadowsocks protocol server.
 
 =head1 VERSION
 
-Version 0.9.0
+Version 0.9.1
 
 =head1 SYNOPSIS
 
@@ -674,14 +723,15 @@ Version 0.9.0
 	camellia-192-cfb camellia-192-ctr camellia-192-ofb
 	camellia-256-cfb camellia-256-ctr camellia-256-ofb
 	chacha20-ietf chacha20-ietf-poly1305
-	rc4-md5 rc4-sha
+	rabbit
 	rc6-128-cfb rc6-128-ctr rc6-128-ofb
 	rc6-192-cfb rc6-192-ctr rc6-192-ofb
 	rc6-256-cfb rc6-256-ctr rc6-256-ofb
+	spritz
 
 2.The following ciphers deprecated by Shadowsocks are not supported: 
 
-      bf-cfb chacha20 salsa20
+      bf-cfb chacha20 salsa20 rc4-md5
 
 3.The following ciphers recommended by Shadowsocks are not supported yet: 
  
